/***************************************************************************
                          quickview.h  -  description
                             -------------------
    begin                : Sun Jul 14 2002
    copyright            : (C) 2002 by Chris Wallace
    email                : chris.wallace@netflash.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef QUICKVIEW_H
#define QUICKVIEW_H

#include <qwidget.h>
#include <qstring.h>
#include <qlist.h>
#include <qvaluelist.h>
#include <qtimer.h>

#include <kio/jobclasses.h>
#include <kio/job.h>

#include "site.h"
#include "rdfitem.h"

class QToolButton;
class QScrollView;

class SiteHandler;
class SchemeTree;

class QuickViewScroller;
class QuickViewURL;

/**
  *@author Chris Wallace
  */

class QuickView : public QWidget  {
   Q_OBJECT
  public:
    enum Directions {
      Vertical = 1,
      Horizontal = 2
    };

	  QuickView(QWidget *parent=0, const char *name=0);
	  ~QuickView();

    void preview(SiteHandler *sh);

  public slots:
    virtual void show(void);
    virtual void hide(void);

  signals:
    void signalDnDFinished(void);

  protected slots:
    void slotFeedBackClicked(void);
    void slotItemBackClicked(void);
    void slotFeedNextClicked(void);
    void slotItemNextClicked(void);

    void slotTimeout();

    void slotScrollerResized(int, int, int, int);

    void slotRightClickURL(const QString &);
    void slotLeftClickURL(const QString &);

    /* For drag and drop */
    void slotContentsDragEntered(QDragEnterEvent *);
    void slotContentsDroped(QDropEvent *);
    void slotResult(KIO::Job *);
		void slotCopyingDone(KIO::Job *, const KURL &, const KURL &, bool, bool);
    void slotDnDFinished(void);

  protected:
    enum GoToSite {
      FirstSite,
      NextSite,
      PreviousSite,
      LastSite,
      SameSite
    };
    enum GoToItem {
      FirstItem,
      NextItem,
      PreviousItem,
      LastItem,
      SameItem
    };

    void goTo(GoToSite s, GoToItem i);

    void setupHorizontal(bool newitem = true);
    void setupVertical(bool newitem = true);

    void setupCommon(void);

    void setupFading(void);

    void setupToolTip(const QString & title, const QString & desc, const QString & url);

  private:
    QToolButton *mFeedBack;
    QToolButton *mFeedNext;
    QToolButton *mItemBack;
    QToolButton *mItemNext;
    QuickViewScroller *mScroller;
    QuickViewURL *mItem;

    QList<Site> mSites;

    QValueList<RDFItem> mItems;

    QTimer mTimer;

    unsigned int mCurrentItem;
    unsigned int mCurrentSite;

    int mScrollMethod;

    unsigned int mDiff;
    unsigned int mTotal;

     /* Fading Stuff */
    bool mFade;
    unsigned int mFadeSpeed;
    unsigned int mFadeDist; //How far from each end should we start/end our fading
    QColor mBackColour;
    QColor mTextColour;
    QColor mLinkColour;

    SiteHandler *mSiteHandler;
};

#endif
