/***************************************************************************
                          rdfdefaulthandler.cpp  -  description
                             -------------------
    begin                : Sat Jul 13 2002
    copyright            : (C) 2002 by Chris Wallace
    email                : chris.wallace@netflash.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "rdfdefaulthandler.h"

#include <qdom.h>
#include <qxml.h>

#include <iostream>

RDFDefaultHandler::RDFDefaultHandler()
{
}

RDFDefaultHandler::~RDFDefaultHandler()
{
}

bool RDFDefaultHandler::startDocument()
{
  std::cerr << "Warning: Unhandled RDFDefaultHandler::startDocument call" << std::endl;
  return true;
}

bool RDFDefaultHandler::endDocument()
{
  std::cerr << "Warning: Unhandled RDFDefaultHandler::endDocument call" << std::endl;
  return true;
}

bool RDFDefaultHandler::startElement(const QString & , const QString & , const QString & , const QXmlAttributes & )
{
  std::cerr << "Warning: Unhandled RDFDefaultHandler::startElement(...) call" << std::endl;
  return true;
}

bool RDFDefaultHandler::endElement(const QString & , const QString & , const QString & )
{
  std::cerr << "Warning: Unhandled RDFDefaultHandler::endElement(...) call" << std::endl;
  return true;
}

bool RDFDefaultHandler::characters(const QString & )
{
  std::cerr << "Warning: Unhandled RDFDefaultHandler::characters(...) call" << std::endl;
  return true;
}

bool RDFDefaultHandler::warning(const QXmlParseException & )
{
  std::cerr << "Warning: Unhandled RDFDefaultHandler::warning(...) call" << std::endl;
  return true;
}

bool RDFDefaultHandler::error(const QXmlParseException & )
{
  std::cerr << "Warning: Unhandled RDFDefaultHandler::error(...) call" << std::endl;
  return true;
}

bool RDFDefaultHandler::fatalError(const QXmlParseException & )
{
  std::cerr << "Warning: Unhandled RDFDefaultHandler::fatalError(...) call" << std::endl;
  return true;
}

bool RDFDefaultHandler::comment(const QString & )
{
  std::cerr << "Warning: Unhandled RDFDefaultHandler::comment(...) call" << std::endl;
  return true;
}

bool RDFDefaultHandler::parse(const QString & contents)
{
  if(!startDocument())
    return false;

  QDomDocument doc;
  QString errMsg = QString::null;
  int errLine = 0;
  int errColumn = 0;
  if(!doc.setContent(contents, true, &errMsg, &errLine, &errColumn)) {
    QXmlParseException e(errMsg, errLine, errColumn);
    if(!fatalError(e)) {
      endDocument();
      return false;
    }
  }

  if(!parseNode(doc))
    return false;

  if(!endDocument())
    return false;

  return true;
}

bool RDFDefaultHandler::parseNode(QDomNode & node)
{
  if(node.isNull())
    return true;

  if(node.isComment()) {
    QDomComment c = node.toComment();
    if(!c.isNull())
      return comment(c.data());
    else
      return false;
  }
  else if(node.isElement()) {
    QDomElement e = node.toElement();
    if(!e.isNull()) {
      QXmlAttributes attrs;
      QDomNamedNodeMap nmap = e.attributes();
      for(unsigned int i=0; i<nmap.count(); i++) {
        QDomNode n = nmap.item(i);
        if(n.isAttr()) {
          QDomAttr a = n.toAttr();
          if(!a.isNull()) {
            attrs.append(a.name(), a.namespaceURI(), QString::null, a.value());
          }
        }
      }

      if(!startElement(e.namespaceURI(), e.localName(), e.tagName(), attrs))
        return false;

      if(e.hasChildNodes()) {
        QDomNodeList kids = e.childNodes();
        for(unsigned int i = 0; i<kids.count(); i++) {
          QDomNode kn = kids.item(i);
          if(!kn.isNull())
            parseNode(kn);
        }
      }
      
      if(!endElement(e.namespaceURI(), e.localName(), e.tagName()))
        return false;
    }
    else
      return false;
  }
  else if(node.isText()) {
    QDomText t = node.toText();
    if(!t.isNull()) {
      if(!characters(t.data()))
        return false;
    }
    else
      return false;
  }
  else if(node.isDocument()) {
    QDomDocument d = node.toDocument();
    if(!d.isNull()) {
      if(d.hasChildNodes()) {
        QDomNodeList kids = d.childNodes();
        for(unsigned int i = 0; i<kids.count(); i++) {
          QDomNode kn = kids.item(i);
          if(!kn.isNull())
            parseNode(kn);
        }
      }
    }
  }
  else if(node.isDocumentFragment()) {
    QDomDocumentFragment f = node.toDocumentFragment();
    if(!f.isNull()) {
      if(f.hasChildNodes()) {
        QDomNodeList kids = f.childNodes();
        for(unsigned int i = 0; i<kids.count(); i++) {
          QDomNode kn = kids.item(i);
          if(!kn.isNull())
            parseNode(kn);
        }
      }
    }
  }
  else if(node.isEntity()) {
    QDomEntity e = node.toEntity();
    if(!e.isNull()) {
      if(e.hasChildNodes()) {
        QDomNodeList kids = e.childNodes();
        for(unsigned int i=0; i<kids.count(); i++) {
          QDomNode kn = kids.item(i);
          if(!kn.isNull())
            parseNode(kn);
        }
      }
    }
  }
  else if(node.isDocumentType()) {
    QDomDocumentType t = node.toDocumentType();
    if(!t.isNull()) {
      if(t.hasChildNodes()) {
        QDomNodeList kids = t.childNodes();
        for(unsigned int i=0; i<kids.count(); i++) {
          QDomNode kn = kids.item(i);
          if(!kn.isNull())
            parseNode(kn);
        }
      }
    }
  }
  else {
    QXmlParseException e(QString("Unknown node type"), -1, -1);
    warning(e);
  }

  return true;
}
